function [f,d,b,L] = run_EM(X,m)
% 
% Runs EM loop until the marginal log-likelihood of observing X given current
% estimate of parameters is idempotent as defined by a fixed tolerance
%
% Input parameters:
%   X HxWxN matrix, N images containing the villain's face
%   m use first m <= N images in X for maximizing likelihood
%
% Output parameters:
%   f estimate of the image of the face
%   d estimate of the displacements of the face for each image
%   b estimate of the background
%   L the marginal log-likelihood of observing image set X 

% Face width
tolerance = 0.001;
max_iters = 20;
w = 36;
H = size(X,1);
W = size(X,2);
dmax = W-w+1;
iter = 1;

oldL = -Inf;

% randomize EM starting point using FIXED seed
f = rand(H,w)*256;
b = rand(1,1)*256;
sigma = rand(1,1)*256^2;
Pd = rand(dmax,1);
Pd = Pd/sum(Pd);
d = [];

while 1
    %E-step
    alpha = e_step(X,f,b,sigma,Pd,m);
    %M-step        
    [f,b,sigma,Pd] = m_step(X,alpha,f,b,sigma,m);
    
    % estimate marginal log-likelihood of observing X given current parameters
    %
    % 
    % 

    % check to see if marginal log-likelihood is idempotent or stop after 20
    % iterations 
    if (L-oldL) < tolerance || iter > max_iters
        break;
    end
    oldL = L;
    iter = iter+1;
end